@@ -18,7 +18,8 @@
 
 import datetime
 import munch
-import six
+
+from openstack import resource
 
 _IMAGE_FIELDS = (
     'checksum',
@@ -38,15 +39,19 @@ _SERVER_FIELDS = (
     'addresses',
     'adminPass',
     'created',
+    'description',
     'key_name',
     'metadata',
     'networks',
+    'personality',
     'private_v4',
     'public_v4',
     'public_v6',
+    'server_groups',
     'status',
     'updated',
     'user_id',
+    'tags',
 )
 
 _KEYPAIR_FIELDS = (
@@ -111,7 +116,7 @@ def _split_filters(obj_name='', filters=None, **kwargs):
 
 
 def _to_bool(value):
-    if isinstance(value, six.string_types):
+    if isinstance(value, str):
         if not value:
             return False
         prospective = value.lower().capitalize()
@@ -134,7 +139,7 @@ def _pop_or_get(resource, key, default, strict):
         return resource.get(key, default)
 
 
-class Normalizer(object):
+class Normalizer:
     '''Mix-in class to provide the normalization functions.
 
     This is in a separate class just for on-disk source code organization
@@ -263,11 +268,18 @@ class Normalizer(object):
         return ret
 
     def _normalize_image(self, image):
-        new_image = munch.Munch(
-            location=self._get_current_location(project_id=image.get('owner')))
+        if isinstance(image, resource.Resource):
+            image = image.to_dict(ignore_none=True, original_names=True)
+            location = image.pop(
+                'location',
+                self._get_current_location(project_id=image.get('owner')))
+        else:
+            location = self._get_current_location(
+                project_id=image.get('owner'))
+            # This copy is to keep things from getting epically weird in tests
+            image = image.copy()
 
-        # This copy is to keep things from getting epically weird in tests
-        image = image.copy()
+        new_image = munch.Munch(location=location)
 
         # Discard noise
         self._remove_novaclient_artifacts(image)
@@ -315,7 +327,7 @@ class Normalizer(object):
         new_image['is_protected'] = protected
         new_image['locations'] = image.pop('locations', [])
 
-        metadata = image.pop('metadata', {})
+        metadata = image.pop('metadata', {}) or {}
         for key, val in metadata.items():
             properties.setdefault(key, val)
 
@@ -375,6 +387,9 @@ class Normalizer(object):
         ret['description'] = group.pop('description')
         ret['properties'] = group
 
+        if self._use_neutron_secgroups():
+            ret['stateful'] = group.pop('stateful', True)
+
         # Backwards compat with Neutron
         if not self.strict_mode:
             ret['tenant_id'] = project_id
@@ -451,7 +466,8 @@ class Normalizer(object):
     def _normalize_server(self, server):
         ret = munch.Munch()
         # Copy incoming server because of shared dicts in unittests
-        server = server.copy()
+        # Wrap the copy in munch so that sub-dicts are properly munched
+        server = munch.Munch(server)
 
         self._remove_novaclient_artifacts(server)
 
@@ -460,30 +476,44 @@ class Normalizer(object):
 
         server['flavor'].pop('links', None)
         ret['flavor'] = server.pop('flavor')
+        # From original_names from sdk
+        server.pop('flavorRef', None)
 
         # OpenStack can return image as a string when you've booted
         # from volume
-        if str(server['image']) != server['image']:
-            server['image'].pop('links', None)
-        ret['image'] = server.pop('image')
+        image = server.pop('image', None)
+        if str(image) != image:
+            image = munch.Munch(id=image['id'])
+
+        ret['image'] = image
+        # From original_names from sdk
+        server.pop('imageRef', None)
+        # From original_names from sdk
+        ret['block_device_mapping'] = server.pop('block_device_mapping_v2', {})
 
         project_id = server.pop('tenant_id', '')
         project_id = server.pop('project_id', project_id)
 
         az = _pop_or_get(
             server, 'OS-EXT-AZ:availability_zone', None, self.strict_mode)
-        ret['location'] = self._get_current_location(
-            project_id=project_id, zone=az)
+        # the server resource has this already, but it's missing az info
+        # from the resource.
+        # TODO(mordred) create_server is still normalizing servers that aren't
+        # from the resource layer.
+        ret['location'] = server.pop(
+            'location', self._get_current_location(
+                project_id=project_id, zone=az))
 
         # Ensure volumes is always in the server dict, even if empty
         ret['volumes'] = _pop_or_get(
             server, 'os-extended-volumes:volumes_attached',
             [], self.strict_mode)
 
-        config_drive = server.pop('config_drive', False)
+        config_drive = server.pop(
+            'has_config_drive', server.pop('config_drive', False))
         ret['has_config_drive'] = _to_bool(config_drive)
 
-        host_id = server.pop('hostId', None)
+        host_id = server.pop('hostId', server.pop('host_id', None))
         ret['host_id'] = host_id
 
         ret['progress'] = _pop_int(server, 'progress')
@@ -496,7 +526,19 @@ class Normalizer(object):
                 'OS-EXT-STS:task_state',
                 'OS-EXT-STS:vm_state',
                 'OS-SRV-USG:launched_at',
-                'OS-SRV-USG:terminated_at'):
+                'OS-SRV-USG:terminated_at',
+                'OS-EXT-SRV-ATTR:hypervisor_hostname',
+                'OS-EXT-SRV-ATTR:instance_name',
+                'OS-EXT-SRV-ATTR:user_data',
+                'OS-EXT-SRV-ATTR:host',
+                'OS-EXT-SRV-ATTR:hostname',
+                'OS-EXT-SRV-ATTR:kernel_id',
+                'OS-EXT-SRV-ATTR:launch_index',
+                'OS-EXT-SRV-ATTR:ramdisk_id',
+                'OS-EXT-SRV-ATTR:reservation_id',
+                'OS-EXT-SRV-ATTR:root_device_name',
+                'OS-SCH-HNT:scheduler_hints',
+        ):
             short_key = key.split(':')[1]
             ret[short_key] = _pop_or_get(server, key, None, self.strict_mode)
 
@@ -523,7 +565,10 @@ class Normalizer(object):
             ret['config_drive'] = config_drive
             ret['project_id'] = project_id
             ret['tenant_id'] = project_id
-            ret['region'] = self.config.region_name
+            # TODO(efried): This is hardcoded to 'compute' because this method
+            # should only ever be used by the compute proxy. (That said, it
+            # doesn't appear to be used at all, so can we get rid of it?)
+            ret['region'] = self.config.get_region_name('compute')
             ret['cloud'] = self.config.name
             ret['az'] = az
             for key, val in ret['properties'].items():
@@ -939,6 +984,9 @@ class Normalizer(object):
             location=self._get_current_location(),
         )
 
+        if not self.strict_mode:
+            ret['uuid'] = c_id
+
         for key in (
                 'status',
                 'cluster_template_id',
@@ -948,7 +996,8 @@ class Normalizer(object):
                 'create_timeout',
                 'node_count',
                 'name'):
-            ret[key] = coe_cluster.pop(key)
+            if key in coe_cluster:
+                ret[key] = coe_cluster.pop(key)
 
         ret['properties'] = coe_cluster
         return ret
@@ -1062,7 +1111,10 @@ class Normalizer(object):
 
     def _normalize_stack(self, stack):
         """Normalize Heat Stack"""
-        stack = stack.copy()
+        if isinstance(stack, resource.Resource):
+            stack = stack.to_dict(ignore_none=True, original_names=True)
+        else:
+            stack = stack.copy()
 
         # Discard noise
         self._remove_novaclient_artifacts(stack)
@@ -1072,7 +1124,10 @@ class Normalizer(object):
         stack.pop('status', None)
         stack.pop('identifier', None)
 
-        stack_status = stack.pop('stack_status')
+        stack_status = None
+
+        stack_status = stack.pop('stack_status', None) or \
+            stack.pop('status', None)
         (action, status) = stack_status.split('_', 1)
 
         ret = munch.Munch(
@@ -1101,13 +1156,13 @@ class Normalizer(object):
                 ('tempate_description', 'template_description'),
                 ('timeout_mins', 'timeout_mins'),
                 ('tags', 'tags')):
-            value = stack.pop(old_name, None)
+            value = stack.get(old_name, None)
             ret[new_name] = value
             if not self.strict_mode:
                 ret[old_name] = value
         ret['identifier'] = '{name}/{id}'.format(
             name=ret['name'], id=ret['id'])
-        ret['properties'] = stack
+        # ret['properties'] = stack
         return ret
 
     def _normalize_machines(self, machines):
@@ -1119,13 +1174,14 @@ class Normalizer(object):
 
     def _normalize_machine(self, machine):
         """Normalize Ironic Machine"""
-        machine = machine.copy()
+        if isinstance(machine, resource.Resource):
+            machine = machine._to_munch()
+        else:
+            machine = machine.copy()
 
         # Discard noise
         self._remove_novaclient_artifacts(machine)
 
-        # TODO(mordred) Normalize this resource
-
         return machine
 
     def _normalize_roles(self, roles):
@@ -1145,3 +1201,37 @@ class Normalizer(object):
             location=self._get_identity_location(),
             properties={},
         )
+
+    def _normalize_containers(self, containers):
+        """Normalize Swift Containers"""
+        ret = []
+        for container in containers:
+            ret.append(self._normalize_container(container))
+        return ret
+
+    def _normalize_container(self, container):
+        """Normalize Swift Container."""
+
+        return munch.Munch(
+            name=container.get('name'),
+            bytes=container.get('bytes'),
+            count=container.get('count'),
+        )
+
+    def _normalize_objects(self, objects):
+        """Normalize Swift Objects"""
+        ret = []
+        for object in objects:
+            ret.append(self._normalize_object(object))
+        return ret
+
+    def _normalize_object(self, object):
+        """Normalize Swift Object."""
+
+        return munch.Munch(
+            name=object.get('name'),
+            bytes=object.get('_bytes'),
+            content_type=object.get('content_type'),
+            hash=object.get('_hash'),
+            last_modified=object.get('_last_modified'),
+        )
