@@ -9,7 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import warnings
 
+from openstack.compute.v2 import aggregate as _aggregate
 from openstack.compute.v2 import availability_zone
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor as _flavor
@@ -18,22 +20,219 @@ from openstack.compute.v2 import image as _image
 from openstack.compute.v2 import keypair as _keypair
 from openstack.compute.v2 import limits
 from openstack.compute.v2 import server as _server
+from openstack.compute.v2 import server_diagnostics as _server_diagnostics
 from openstack.compute.v2 import server_group as _server_group
 from openstack.compute.v2 import server_interface as _server_interface
+from openstack.compute.v2 import (
+    server_remote_console as _server_remote_console)
 from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import service as _service
 from openstack.compute.v2 import volume_attachment as _volume_attachment
+from openstack.network.v2 import security_group as _sg
 from openstack import proxy
 from openstack import resource
 
 
 class Proxy(proxy.Proxy):
 
-    Extension = extension.Extension
-    Flavor = _flavor.Flavor
-    FlavorDetail = _flavor.FlavorDetail
-    Server = _server.Server
-    ServerDetail = _server.ServerDetail
+    def find_extension(self, name_or_id, ignore_missing=True):
+        """Find a single extension
+
+        :param name_or_id: The name or ID of an extension.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the resource does not exist.
+                    When set to ``True``, None will be returned when
+                    attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.extension.Extension` or
+                  None
+        """
+        return self._find(extension.Extension, name_or_id,
+                          ignore_missing=ignore_missing)
+
+    def extensions(self):
+        """Retrieve a generator of extensions
+
+        :returns: A generator of extension instances.
+        :rtype: :class:`~openstack.compute.v2.extension.Extension`
+        """
+        return self._list(extension.Extension)
+
+    def find_flavor(self, name_or_id, ignore_missing=True):
+        """Find a single flavor
+
+        :param name_or_id: The name or ID of a flavor.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the resource does not exist.
+                    When set to ``True``, None will be returned when
+                    attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor` or None
+        """
+        return self._find(_flavor.Flavor, name_or_id,
+                          ignore_missing=ignore_missing)
+
+    def create_flavor(self, **attrs):
+        """Create a new flavor from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+                           a :class:`~openstack.compute.v2.flavor.Flavor`,
+                           comprised of the properties on the Flavor class.
+
+        :returns: The results of flavor creation
+        :rtype: :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        return self._create(_flavor.Flavor, **attrs)
+
+    def delete_flavor(self, flavor, ignore_missing=True):
+        """Delete a flavor
+
+        :param flavor: The value can be either the ID of a flavor or a
+                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the flavor does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent flavor.
+
+        :returns: ``None``
+        """
+        self._delete(_flavor.Flavor, flavor, ignore_missing=ignore_missing)
+
+    def get_flavor(self, flavor):
+        """Get a single flavor
+
+        :param flavor: The value can be the ID of a flavor or a
+                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
+
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_flavor.Flavor, flavor)
+
+    def flavors(self, details=True, **query):
+        """Return a generator of flavors
+
+        :param bool details: When ``True``, returns
+            :class:`~openstack.compute.v2.flavor.FlavorDetail` objects,
+            otherwise :class:`~openstack.compute.v2.flavor.Flavor`.
+            *Default: ``True``*
+        :param kwargs query: Optional query parameters to be sent to limit
+                                 the flavors being returned.
+
+        :returns: A generator of flavor objects
+        """
+        flv = _flavor.FlavorDetail if details else _flavor.Flavor
+        return self._list(flv, **query)
+
+    def aggregates(self, **query):
+        """Return a generator of aggregate
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the aggregates being returned.
+
+        :returns: A generator of aggregate
+        :rtype: class: `~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._list(_aggregate.Aggregate, **query)
+
+    def get_aggregate(self, aggregate):
+        """Get a single host aggregate
+
+        :param image: The value can be the ID of an aggregate or a
+                      :class:`~openstack.compute.v2.aggregate.Aggregate`
+                      instance.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_aggregate.Aggregate, aggregate)
+
+    def create_aggregate(self, **attrs):
+        """Create a new host aggregate from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create a
+                           :class:`~openstack.compute.v2.aggregate.Aggregate`,
+                           comprised of the properties on the Aggregate class.
+
+        :returns: The results of aggregate creation
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._create(_aggregate.Aggregate, **attrs)
+
+    def update_aggregate(self, aggregate, **attrs):
+        """Update a host aggregate
+
+        :param server: Either the ID of a host aggregate or a
+                       :class:`~openstack.compute.v2.aggregate.Aggregate`
+                       instance.
+        :attrs kwargs: The attributes to update on the aggregate represented
+                       by ``aggregate``.
+
+        :returns: The updated aggregate
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._update(_aggregate.Aggregate, aggregate, **attrs)
+
+    def delete_aggregate(self, aggregate, ignore_missing=True):
+        """Delete a host aggregate
+
+        :param keypair: The value can be either the ID of an aggregate or a
+                        :class:`~openstack.compute.v2.aggregate.Aggregate`
+                        instance.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the aggregate does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent aggregate.
+
+        :returns: ``None``
+        """
+        self._delete(_aggregate.Aggregate, aggregate,
+                     ignore_missing=ignore_missing)
+
+    def add_host_to_aggregate(self, aggregate, host):
+        """Adds a host to an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param str host: The host to add to the aggregate
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.add_host(self, host)
+
+    def remove_host_from_aggregate(self, aggregate, host):
+        """Removes a host from an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param str host: The host to remove from the aggregate
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.remove_host(self, host)
+
+    def set_aggregate_metadata(self, aggregate, metadata):
+        """Creates or replaces metadata for an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param dict metadata: Metadata key and value pairs. The maximum
+                    size for each metadata key and value pair
+                    is 255 bytes.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.set_metadata(self, metadata)
 
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
@@ -80,16 +279,18 @@ class Proxy(proxy.Proxy):
         """Return a generator of images
 
         :param bool details: When ``True``, returns
-            :class:`~openstack.compute.v2.image.ImageDetail` objects,
-            otherwise :class:`~openstack.compute.v2.image.Image`.
+            :class:`~openstack.compute.v2.image.Image` objects with all
+            available properties, otherwise only basic properties are returned.
             *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of image objects
         """
-        img = _image.ImageDetail if details else _image.Image
-        return self._list(img, paginated=True, **query)
+        warnings.warn('This API is deprecated and may disappear shortly',
+                      DeprecationWarning)
+        base_path = '/images/detail' if details else None
+        return self._list(_image.Image, base_path=base_path, **query)
 
     def _get_base_resource(self, res, base):
         # Metadata calls for Image and Server can work for both those
@@ -104,9 +305,7 @@ class Proxy(proxy.Proxy):
         """Return a dictionary of metadata for an image
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
 
         :returns: A :class:`~openstack.compute.v2.image.Image` with only the
                   image's metadata. All keys and values are Unicode text.
@@ -121,9 +320,7 @@ class Proxy(proxy.Proxy):
         """Update metadata for an image
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
         :param kwargs metadata: Key/value pairs to be updated in the image's
                                 metadata. No other metadata is modified
                                 by this call. All keys and values are stored
@@ -144,9 +341,7 @@ class Proxy(proxy.Proxy):
         Note: This method will do a HTTP DELETE request for every key in keys.
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
         :param keys: The keys to delete.
 
         :rtype: ``None``
@@ -209,13 +404,15 @@ class Proxy(proxy.Proxy):
         return self._find(_keypair.Keypair, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def keypairs(self):
+    def keypairs(self, **query):
         """Return a generator of keypairs
 
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
         :returns: A generator of keypair objects
         :rtype: :class:`~openstack.compute.v2.keypair.Keypair`
         """
-        return self._list(_keypair.Keypair, paginated=False)
+        return self._list(_keypair.Keypair, **query)
 
     def get_limits(self):
         """Retrieve limits that are applied to the project's account
@@ -227,6 +424,18 @@ class Proxy(proxy.Proxy):
         """
         return self._get(limits.Limits)
 
+    def create_server(self, **attrs):
+        """Create a new server from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+                           a :class:`~openstack.compute.v2.server.Server`,
+                           comprised of the properties on the Server class.
+
+        :returns: The results of server creation
+        :rtype: :class:`~openstack.compute.v2.server.Server`
+        """
+        return self._create(_server.Server, **attrs)
+
     def delete_server(self, server, ignore_missing=True, force=False):
         """Delete a server
 
@@ -246,47 +455,50 @@ class Proxy(proxy.Proxy):
             server = self._get_resource(_server.Server, server)
             server.force_delete(self)
         else:
-            self._generated_delete_server(
-                server, ignore_missing=ignore_missing)
+            self._delete(_server.Server, server, ignore_missing=ignore_missing)
 
-    def servers(self, details=True, **query):
+    def find_server(self, name_or_id, ignore_missing=True):
+        """Find a single server
+
+        :param name_or_id: The name or ID of a server.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the resource does not exist.
+                    When set to ``True``, None will be returned when
+                    attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.server.Server` or None
+        """
+        return self._find(_server.Server, name_or_id,
+                          ignore_missing=ignore_missing)
+
+    def get_server(self, server):
+        """Get a single server
+
+        :param server: The value can be the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: One :class:`~openstack.compute.v2.server.Server`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_server.Server, server)
+
+    def servers(self, details=True, all_projects=False, **query):
         """Retrieve a generator of servers
 
         :param bool details: When set to ``False``
-                    :class:`~openstack.compute.v2.server.Server` instances
-                    will be returned. The default, ``True``, will cause
-                    :class:`~openstack.compute.v2.server.ServerDetail`
-                    instances to be returned.
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-            the servers being returned.  Available parameters include:
-
-            * changes_since: A time/date stamp for when the server last changed
-                             status.
-            * image: An image resource or ID.
-            * flavor: A flavor resource or ID.
-            * name: Name of the server as a string.  Can be queried with
-                    regular expressions. The regular expression
-                    ?name=bob returns both bob and bobb.  If you must match on
-                    only bob, you can use a regular expression that
-                    matches the syntax of the underlying database server that
-                    is implemented for Compute, such as MySQL or PostgreSQL.
-            * status: Value of the status of the server so that you can filter
-                    on "ACTIVE" for example.
-            * host: Name of the host as a string.
-            * limit: Requests a specified page size of returned items from the
-                     query.  Returns a number of items up to the specified
-                     limit value. Use the limit parameter to make an initial
-                     limited request and use the ID of the last-seen item from
-                     the response as the marker parameter value in a subsequent
-                     limited request.
-            * marker: Specifies the ID of the last-seen item. Use the limit
-                      parameter to make an initial limited request and use the
-                      ID of the last-seen item from the response as the marker
-                      parameter value in a subsequent limited request.
+            instances with only basic data will be returned. The default,
+            ``True``, will cause instances with full data to be returned.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the servers being returned. Available parameters can be seen
+            under https://docs.openstack.org/api-ref/compute/#list-servers
 
         :returns: A generator of server instances.
         """
-        return self._generated_servers(details=details, **query)
+        if all_projects:
+            query['all_projects'] = True
+        base_path = '/servers/detail' if details else None
+        return self._list(_server.Server, base_path=base_path, **query)
 
     def update_server(self, server, **attrs):
         """Update a server
@@ -414,39 +626,58 @@ class Proxy(proxy.Proxy):
         server = self._get_resource(_server.Server, server)
         server.revert_resize(self)
 
-    def create_server_image(self, server, name, metadata=None):
+    def create_server_image(self, server, name, metadata=None, wait=False,
+                            timeout=120):
         """Create an image from a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param str name: The name of the image to be created.
         :param dict metadata: A dictionary of metadata to be set on the image.
 
-        :returns: None
+        :returns: :class:`~openstack.image.v2.image.Image` object.
         """
         server = self._get_resource(_server.Server, server)
-        server.create_image(self, name, metadata)
+        image_id = server.create_image(self, name, metadata)
+
+        self._connection.list_images.invalidate(self)
+        image = self._connection.get_image(image_id)
+
+        if not wait:
+            return image
+        return self._connection.wait_for_image(image, timeout=timeout)
+
+    def fetch_server_security_groups(self, server):
+        """Fetch security groups with details for a server.
+
+        :param server: Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: updated :class:`~openstack.compute.v2.server.Server` instance
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.fetch_security_groups(self)
 
     def add_security_group_to_server(self, server, security_group):
         """Add a security group to a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-        :param security_group: Either the ID of a security group or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param security_group: Either the ID, Name of a security group or a
             :class:`~openstack.network.v2.security_group.SecurityGroup`
             instance.
 
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource.Resource._get_id(security_group)
-        server.add_security_group(self, security_group_id)
+        security_group = self._get_resource(_sg.SecurityGroup, security_group)
+        server.add_security_group(self, security_group.name)
 
     def remove_security_group_from_server(self, server, security_group):
         """Remove a security group from a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param security_group: Either the ID of a security group or a
             :class:`~openstack.network.v2.security_group.SecurityGroup`
             instance.
@@ -454,8 +685,8 @@ class Proxy(proxy.Proxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource.Resource._get_id(security_group)
-        server.remove_security_group(self, security_group_id)
+        security_group = self._get_resource(_sg.SecurityGroup, security_group)
+        server.remove_security_group(self, security_group.name)
 
     def add_fixed_ip_to_server(self, server, network_id):
         """Adds a fixed IP address to a server instance.
@@ -692,8 +923,33 @@ class Proxy(proxy.Proxy):
         server = self._get_resource(_server.Server, server)
         return server.get_console_output(self, length=length)
 
-    def wait_for_server(self, server, status='ACTIVE', failures=['ERROR'],
+    def wait_for_server(self, server, status='ACTIVE', failures=None,
                         interval=2, wait=120):
+        """Wait for a server to be in a particular status.
+
+        :param server:
+            The :class:`~openstack.compute.v2.server.Server` to wait on
+            to reach the specified status.
+        :type server: :class:`~openstack.compute.v2.server.Server`:
+        :param status: Desired status.
+        :param failures:
+            Statuses that would be interpreted as failures.
+        :type failures: :py:class:`list`
+        :param int interval:
+            Number of seconds to wait before to consecutive checks.
+            Default to 2.
+        :param int wait:
+            Maximum number of seconds to wait before the change.
+            Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to the desired status failed to occur in specified seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+                 has transited to one of the failure statuses.
+        :raises: :class:`~AttributeError` if the resource does not have a
+                ``status`` attribute.
+        """
+        failures = ['ERROR'] if failures is None else failures
         return resource.wait_for_status(
             self, server, status, failures, interval, wait)
 
@@ -777,7 +1033,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(_server_interface.ServerInterface, paginated=False,
+        return self._list(_server_interface.ServerInterface,
                           server_id=server_id)
 
     def server_ips(self, server, network_label=None):
@@ -792,7 +1048,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.server_ip.ServerIP`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(server_ip.ServerIP, paginated=False,
+        return self._list(server_ip.ServerIP,
                           server_id=server_id, network_label=network_label)
 
     def availability_zones(self, details=False):
@@ -806,12 +1062,11 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.availability_zone.\
                         AvailabilityZone`
         """
-        if details:
-            az = availability_zone.AvailabilityZoneDetail
-        else:
-            az = availability_zone.AvailabilityZone
+        base_path = '/os-availability-zone/detail' if details else None
 
-        return self._list(az, paginated=False)
+        return self._list(
+            availability_zone.AvailabilityZone,
+            base_path=base_path)
 
     def get_server_metadata(self, server):
         """Return a dictionary of metadata for a server
@@ -834,9 +1089,7 @@ class Proxy(proxy.Proxy):
         """Update metadata for a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` or
-                       :class:`~openstack.compute.v2.server.ServerDetail`
-                       instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param kwargs metadata: Key/value pairs to be updated in the server's
                                 metadata. No other metadata is modified
                                 by this call. All keys and values are stored
@@ -857,9 +1110,7 @@ class Proxy(proxy.Proxy):
         Note: This method will do a HTTP DELETE request for every key in keys.
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` or
-                       :class:`~openstack.compute.v2.server.ServerDetail`
-                       instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param keys: The keys to delete
 
         :rtype: ``None``
@@ -929,31 +1180,27 @@ class Proxy(proxy.Proxy):
     def server_groups(self, **query):
         """Return a generator of server groups
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of ServerGroup objects
         :rtype: :class:`~openstack.compute.v2.server_group.ServerGroup`
         """
-        return self._list(_server_group.ServerGroup, paginated=False, **query)
+        return self._list(_server_group.ServerGroup, **query)
 
-    def hypervisors(self, details=False):
+    def hypervisors(self, details=False, **query):
         """Return a generator of hypervisor
 
         :param bool details: When set to the default, ``False``,
-                    :class:`~openstack.compute.v2.hypervisor.Hypervisor`
-                    instances will be returned. ``True`` will cause
-                    :class:`~openstack.compute.v2.hypervisor.HypervisorDetail`
-                    instances to be returned.
+            :class:`~openstack.compute.v2.hypervisor.Hypervisor`
+            instances will be returned with only basic information populated.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
         :returns: A generator of hypervisor
         :rtype: class: `~openstack.compute.v2.hypervisor.Hypervisor`
         """
-        if details:
-            hypervisor = _hypervisor.HypervisorDetail
-        else:
-            hypervisor = _hypervisor.Hypervisor
-
-        return self._list(hypervisor, paginated=False)
+        base_path = '/os-hypervisors/detail' if details else None
+        return self._list(_hypervisor.Hypervisor, base_path=base_path, **query)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
@@ -1032,7 +1279,7 @@ class Proxy(proxy.Proxy):
         :rtype: class: `~openstack.compute.v2.service.Service`
         """
 
-        return self._list(_service.Service, paginated=False)
+        return self._list(_service.Service)
 
     def create_volume_attachment(self, server, **attrs):
         """Create a new volume attachment from attributes
@@ -1154,7 +1401,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(_volume_attachment.VolumeAttachment, paginated=False,
+        return self._list(_volume_attachment.VolumeAttachment,
                           server_id=server_id)
 
     def migrate_server(self, server):
@@ -1195,3 +1442,81 @@ class Proxy(proxy.Proxy):
             self, host,
             force=force,
             block_migration=block_migration)
+
+    def wait_for_delete(self, res, interval=2, wait=120):
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait)
+
+    def get_server_diagnostics(self, server):
+        """Get a single server diagnostics
+
+        :param server: This parameter need to be specified when ServerInterface
+                       ID is given as value. It can be either the ID of a
+                       server or a :class:`~openstack.compute.v2.server.Server`
+                       instance that the interface belongs to.
+
+        :returns: One
+            :class:`~openstack.compute.v2.server_diagnostics.ServerDiagnostics`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        server_id = self._get_resource(_server.Server, server).id
+        return self._get(_server_diagnostics.ServerDiagnostics,
+                         server_id=server_id, requires_id=False)
+
+    def create_server_remote_console(self, server, **attrs):
+        """Create a remote console on the server.
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :returns: One
+            :class:`~openstack.compute.v2.server_remote_console.
+                    ServerRemoteConsole`
+        """
+        server_id = resource.Resource._get_id(server)
+        return self._create(_server_remote_console.ServerRemoteConsole,
+                            server_id=server_id, **attrs)
+
+    def _get_cleanup_dependencies(self):
+        return {
+            'compute': {
+                'before': ['block_storage', 'network', 'identity']
+            }
+        }
+
+    def _service_cleanup(self, dry_run=True, client_status_queue=None,
+                         identified_resources=None,
+                         filters=None, resource_evaluation_fn=None):
+        servers = []
+        for obj in self.servers():
+            need_delete = self._service_cleanup_del_res(
+                self.delete_server,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
+            if not dry_run and need_delete:
+                # In the dry run we identified, that server will go. To propely
+                # identify consequences we need to tell others, that the port
+                # will disappear as well
+                for port in self._connection.network.ports(device_id=obj.id):
+                    identified_resources[port.id] = port
+                servers.append(obj)
+
+        # We actually need to wait for servers to really disappear, since they
+        # might be still holding ports on the subnet
+        for server in servers:
+            self.wait_for_delete(server)
