@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.compute import compute_service
 from openstack import resource
 
 
@@ -71,14 +70,14 @@ class RateLimit(resource.Resource):
 class Limits(resource.Resource):
     base_path = "/limits"
     resource_key = "limits"
-    service = compute_service.ComputeService()
 
-    allow_get = True
+    allow_fetch = True
 
     absolute = resource.Body("absolute", type=AbsoluteLimits)
     rate = resource.Body("rate", type=list, list_type=RateLimit)
 
-    def get(self, session, requires_id=False, error_message=None):
+    def fetch(self, session, requires_id=False, error_message=None,
+              base_path=None):
         """Get the Limits resource.
 
         :param session: The session to use for making this request.
@@ -89,5 +88,6 @@ class Limits(resource.Resource):
         """
         # TODO(mordred) We shouldn't have to subclass just to declare
         # requires_id = False.
-        return super(Limits, self).get(
-            session=session, requires_id=False, error_message=error_message)
+        return super(Limits, self).fetch(
+            session=session, requires_id=False, error_message=error_message,
+            base_path=base_path)
