@@ -14,20 +14,47 @@
 
 import copy
 import warnings
+import urllib
 
-from keystoneauth1 import adapter
 from keystoneauth1 import discover
 import keystoneauth1.exceptions.catalog
+from keystoneauth1.loading import adapter as ks_load_adap
 from keystoneauth1 import session as ks_session
 import os_service_types
 import requestsexceptions
-from six.moves import urllib
+try:
+    import statsd
+except ImportError:
+    statsd = None
+try:
+    import prometheus_client
+except ImportError:
+    prometheus_client = None
+try:
+    import influxdb
+except ImportError:
+    influxdb = None
+
 
 from openstack import version as openstack_version
 from openstack import _log
 from openstack.config import _util
 from openstack.config import defaults as config_defaults
 from openstack import exceptions
+from openstack import proxy
+
+
+_logger = _log.setup_logging('openstack')
+
+SCOPE_KEYS = {
+    'domain_id', 'domain_name',
+    'project_id', 'project_name',
+    'system_scope'
+}
+
+
+# Sentinel for nonexistence
+_ENOENT = object()
 
 
 def _make_key(key, service_type):
@@ -38,6 +65,15 @@ def _make_key(key, service_type):
         return "_".join([service_type, key])
 
 
+def _disable_service(config, service_type, reason=None):
+    service_type = service_type.lower().replace('-', '_')
+    key = 'has_{service_type}'.format(service_type=service_type)
+    config[key] = False
+    if reason:
+        d_key = _make_key('disabled_reason', service_type)
+        config[d_key] = reason
+
+
 def _get_implied_microversion(version):
     if not version:
         return
@@ -83,11 +119,114 @@ def from_session(session, name=None, region_name=None,
         app_name=app_name, app_version=app_version)
 
 
-class CloudRegion(object):
+def from_conf(conf, session=None, service_types=None, **kwargs):
+    """Create a CloudRegion from oslo.config ConfigOpts.
+
+    :param oslo_config.cfg.ConfigOpts conf:
+        An oslo.config ConfigOpts containing keystoneauth1.Adapter options in
+        sections named according to project (e.g. [nova], not [compute]).
+        TODO: Current behavior is to use defaults if no such section exists,
+        which may not be what we want long term.
+    :param keystoneauth1.session.Session session:
+        An existing authenticated Session to use. This is currently required.
+        TODO: Load this (and auth) from the conf.
+    :param service_types:
+        A list/set of service types for which to look for and process config
+        opts. If None, all known service types are processed. Note that we will
+        not error if a supplied service type can not be processed successfully
+        (unless you try to use the proxy, of course). This tolerates uses where
+        the consuming code has paths for a given service, but those paths are
+        not exercised for given end user setups, and we do not want to generate
+        errors for e.g. missing/invalid conf sections in those cases. We also
+        don't check to make sure your service types are spelled correctly -
+        caveat implementor.
+    :param kwargs:
+        Additional keyword arguments to be passed directly to the CloudRegion
+        constructor.
+    :raise openstack.exceptions.ConfigException:
+        If session is not specified.
+    :return:
+        An openstack.config.cloud_region.CloudRegion.
+    """
+    if not session:
+        # TODO(mordred) Fill this in - not needed for first stab with nova
+        raise exceptions.ConfigException("A Session must be supplied.")
+    config_dict = kwargs.pop('config', config_defaults.get_defaults())
+    stm = os_service_types.ServiceTypes()
+    for st in stm.all_types_by_service_type:
+        if service_types is not None and st not in service_types:
+            _disable_service(
+                config_dict, st,
+                reason="Not in the list of requested service_types.")
+            continue
+        project_name = stm.get_project_name(st)
+        if project_name not in conf:
+            if '-' in project_name:
+                project_name = project_name.replace('-', '_')
+
+            if project_name not in conf:
+                _disable_service(
+                    config_dict, st,
+                    reason="No section for project '{project}' (service type "
+                           "'{service_type}') was present in the config."
+                    .format(project=project_name, service_type=st))
+                continue
+        opt_dict = {}
+        # Populate opt_dict with (appropriately processed) Adapter conf opts
+        try:
+            ks_load_adap.process_conf_options(conf[project_name], opt_dict)
+        except Exception as e:
+            # NOTE(efried): This is for (at least) a couple of scenarios:
+            # (1) oslo_config.cfg.NoSuchOptError when ksa adapter opts are not
+            #     registered in this section.
+            # (2) TypeError, when opts are registered but bogus (e.g.
+            #     'interface' and 'valid_interfaces' are both present).
+            # We may want to consider (providing a kwarg giving the caller the
+            # option of) blowing up right away for (2) rather than letting them
+            # get all the way to the point of trying the service and having
+            # *that* blow up.
+            reason = ("Encountered an exception attempting to process config "
+                      "for project '{project}' (service type "
+                      "'{service_type}'): {exception}".format(
+                          project=project_name, service_type=st, exception=e))
+            _logger.warning("Disabling service '{service_type}': "
+                            "{reason}".format(service_type=st, reason=reason))
+            _disable_service(config_dict, st, reason=reason)
+            continue
+        # Load them into config_dict under keys prefixed by ${service_type}_
+        for raw_name, opt_val in opt_dict.items():
+            config_name = _make_key(raw_name, st)
+            config_dict[config_name] = opt_val
+    return CloudRegion(
+        session=session, config=config_dict, **kwargs)
+
+
+class CloudRegion:
+    # TODO(efried): Doc the rest of the kwargs
     """The configuration for a Region of an OpenStack Cloud.
 
     A CloudRegion encapsulates the config information needed for connections
     to all of the services in a Region of a Cloud.
+
+    :param str region_name:
+        The default region name for all services in this CloudRegion. If
+        both ``region_name`` and ``config['region_name'] are specified, the
+        kwarg takes precedence. May be overridden for a given ${service}
+        via a ${service}_region_name key in the ``config`` dict.
+    :param dict config:
+        A dict of configuration values for the CloudRegion and its
+        services. The key for a ${config_option} for a specific ${service}
+        should be ${service}_${config_option}. For example, to configure
+        the endpoint_override for the block_storage service, the ``config``
+        dict should contain::
+
+            'block_storage_endpoint_override': 'http://...'
+
+        To provide a default to be used if no service-specific override is
+        present, just use the unprefixed ${config_option} as the service
+        key, e.g.::
+
+            'interface': 'public'
     """
     def __init__(self, name=None, region_name=None, config=None,
                  force_ipv4=False, auth_plugin=None,
@@ -96,10 +235,17 @@ class CloudRegion(object):
                  discovery_cache=None, extra_config=None,
                  cache_expiration_time=0, cache_expirations=None,
                  cache_path=None, cache_class='dogpile.cache.null',
-                 cache_arguments=None, password_callback=None):
+                 cache_arguments=None, password_callback=None,
+                 statsd_host=None, statsd_port=None, statsd_prefix=None,
+                 influxdb_config=None,
+                 collector_registry=None):
         self._name = name
-        self.region_name = region_name
         self.config = _util.normalize_keys(config)
+        # NOTE(efried): For backward compatibility: a) continue to accept the
+        # region_name kwarg; b) make it take precedence over (non-service_type-
+        # specific) region_name set in the config dict.
+        if region_name is not None:
+            self.config['region_name'] = region_name
         self._extra_config = extra_config or {}
         self.log = _log.setup_logging('openstack.config')
         self._force_ipv4 = force_ipv4
@@ -116,6 +262,13 @@ class CloudRegion(object):
         self._cache_class = cache_class
         self._cache_arguments = cache_arguments
         self._password_callback = password_callback
+        self._statsd_host = statsd_host
+        self._statsd_port = statsd_port
+        self._statsd_prefix = statsd_prefix
+        self._statsd_client = None
+        self._influxdb_config = influxdb_config
+        self._influxdb_client = None
+        self._collector_registry = collector_registry
 
         self._service_type_manager = os_service_types.ServiceTypes()
 
@@ -136,7 +289,6 @@ class CloudRegion(object):
     def __eq__(self, other):
         return (
             self.name == other.name
-            and self.region_name == other.region_name
             and self.config == other.config)
 
     def __ne__(self, other):
@@ -159,15 +311,20 @@ class CloudRegion(object):
         Always returns a valid string. It will have name and region_name
         or just one of the two if only one is set, or else 'unknown'.
         """
-        if self.name and self.region_name:
-            return ":".join([self.name, self.region_name])
-        elif self.name and not self.region_name:
+        region_name = self.get_region_name()
+        if self.name and region_name:
+            return ":".join([self.name, region_name])
+        elif self.name and not region_name:
             return self.name
-        elif not self.name and self.region_name:
-            return self.region_name
+        elif not self.name and region_name:
+            return region_name
         else:
             return 'unknown'
 
+    def set_service_value(self, key, service_type, value):
+        key = _make_key(key, service_type)
+        self.config[key] = value
+
     def set_session_constructor(self, session_constructor):
         """Sets the Session constructor."""
         self._session_constructor = session_constructor
@@ -206,6 +363,21 @@ class CloudRegion(object):
                 services.append("_".join(key.split('_')[:-2]))
         return list(set(services))
 
+    def get_enabled_services(self):
+        services = set()
+
+        all_services = [k['service_type'] for k in
+                        self._service_type_manager.services]
+        all_services.extend(k[4:] for k in
+                            self.config.keys() if k.startswith('has_'))
+
+        for srv in all_services:
+            ep = self.get_endpoint_from_catalog(srv)
+            if ep:
+                services.add(srv.replace('-', '_'))
+
+        return services
+
     def get_auth_args(self):
         return self.config.get('auth', {})
 
@@ -229,6 +401,9 @@ class CloudRegion(object):
 
         If none of that works, it returns the value in ``default``.
         '''
+        if service_type is None:
+            return self.config.get(key)
+
         for st in self._service_type_manager.get_all_types(service_type):
             value = self.config.get(_make_key(key, st))
             if value is not None:
@@ -244,12 +419,40 @@ class CloudRegion(object):
                 value = converter(value)
             return value
 
+    def _get_service_config(self, key, service_type):
+        config_dict = self.config.get(key)
+        if not config_dict:
+            return None
+        if not isinstance(config_dict, dict):
+            return config_dict
+
+        for st in self._service_type_manager.get_all_types(service_type):
+            if st in config_dict:
+                return config_dict[st]
+
+    def get_region_name(self, service_type=None):
+        # If a region_name for the specific service_type is configured, use it;
+        # else use the one configured for the CloudRegion as a whole.
+        return self._get_config(
+            'region_name', service_type, fallback_to_unprefixed=True)
+
     def get_interface(self, service_type=None):
         return self._get_config(
             'interface', service_type, fallback_to_unprefixed=True)
 
     def get_api_version(self, service_type):
-        return self._get_config('api_version', service_type)
+        version = self._get_config('api_version', service_type)
+        if version:
+            try:
+                float(version)
+            except ValueError:
+                if 'latest' in version:
+                    warnings.warn(
+                        "You have a configured API_VERSION with 'latest' in"
+                        " it. In the context of openstacksdk this doesn't make"
+                        " any sense.")
+                return None
+        return version
 
     def get_default_microversion(self, service_type):
         return self._get_config('default_microversion', service_type)
@@ -272,11 +475,64 @@ class CloudRegion(object):
         return self._get_config('service_name', service_type)
 
     def get_endpoint(self, service_type):
+        auth = self.config.get('auth', {})
         value = self._get_config('endpoint_override', service_type)
         if not value:
             value = self._get_config('endpoint', service_type)
+        if not value and self.config.get('auth_type') == 'none':
+            # If endpoint is given and we're using the none auth type,
+            # then the endpoint value is the endpoint_override for every
+            # service.
+            value = auth.get('endpoint')
+        if (not value and service_type == 'identity'
+                and SCOPE_KEYS.isdisjoint(set(auth.keys()))):
+            # There are a small number of unscoped identity operations.
+            # Specifically, looking up a list of projects/domains/system to
+            # scope to.
+            value = auth.get('auth_url')
+        # Because of course. Seriously.
+        # We have to override the Rackspace block-storage endpoint because
+        # only v1 is in the catalog but the service actually does support
+        # v2. But the endpoint needs the project_id.
+        service_type = self._service_type_manager.get_service_type(
+            service_type)
+        if (
+            value
+            and self.config.get('profile') == 'rackspace'
+            and service_type == 'block-storage'
+        ):
+            value = value + auth.get('project_id')
         return value
 
+    def get_endpoint_from_catalog(
+            self, service_type, interface=None, region_name=None):
+        """Return the endpoint for a given service as found in the catalog.
+
+        For values respecting endpoint overrides, see
+        :meth:`~openstack.connection.Connection.endpoint_for`
+
+        :param service_type: Service Type of the endpoint to search for.
+        :param interface:
+            Interface of the endpoint to search for. Optional, defaults to
+            the configured value for interface for this Connection.
+        :param region_name:
+            Region Name of the endpoint to search for. Optional, defaults to
+            the configured value for region_name for this Connection.
+
+        :returns: The endpoint of the service, or None if not found.
+        """
+        interface = interface or self.get_interface(service_type)
+        region_name = region_name or self.get_region_name(service_type)
+        session = self.get_session()
+        catalog = session.auth.get_access(session).service_catalog
+        try:
+            return catalog.url_for(
+                service_type=service_type,
+                interface=interface,
+                region_name=region_name)
+        except keystoneauth1.exceptions.catalog.EndpointNotFound:
+            return None
+
     def get_connect_retries(self, service_type):
         return self._get_config('connect_retries', service_type,
                                 fallback_to_unprefixed=True,
@@ -336,6 +592,7 @@ class CloudRegion(object):
                 verify=verify,
                 cert=cert,
                 timeout=self.config.get('api_timeout'),
+                collect_timing=self.config.get('timing'),
                 discovery_cache=self._discovery_cache)
             self.insert_user_agent()
             # Using old keystoneauth with new os-client-config fails if
@@ -404,8 +661,43 @@ class CloudRegion(object):
 
         return version_request
 
+    def get_all_version_data(self, service_type):
+        # Seriously. Don't think about the existential crisis
+        # that is the next line. You'll wind up in cthulhu's lair.
+        service_type = self.get_service_type(service_type)
+        region_name = self.get_region_name(service_type)
+        versions = self.get_session().get_all_version_data(
+            service_type=service_type,
+            interface=self.get_interface(service_type),
+            region_name=region_name,
+        )
+        region_versions = versions.get(region_name, {})
+        interface_versions = region_versions.get(
+            self.get_interface(service_type), {})
+        return interface_versions.get(service_type, [])
+
+    def _get_endpoint_from_catalog(self, service_type, constructor):
+        adapter = constructor(
+            session=self.get_session(),
+            service_type=self.get_service_type(service_type),
+            service_name=self.get_service_name(service_type),
+            interface=self.get_interface(service_type),
+            region_name=self.get_region_name(service_type),
+        )
+        return adapter.get_endpoint()
+
+    def _get_hardcoded_endpoint(self, service_type, constructor):
+        endpoint = self._get_endpoint_from_catalog(
+            service_type, constructor)
+        if not endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
+            if not endpoint.endswith('/'):
+                endpoint += '/'
+            endpoint = urllib.parse.urljoin(endpoint, 'v2.0')
+        return endpoint
+
     def get_session_client(
-            self, service_type, version=None, constructor=adapter.Adapter,
+            self, service_type, version=None,
+            constructor=proxy.Proxy,
             **kwargs):
         """Return a prepped keystoneauth Adapter for a given service.
 
@@ -422,47 +714,48 @@ class CloudRegion(object):
         """
         version_request = self._get_version_request(service_type, version)
 
+        kwargs.setdefault('region_name', self.get_region_name(service_type))
         kwargs.setdefault('connect_retries',
                           self.get_connect_retries(service_type))
         kwargs.setdefault('status_code_retries',
                           self.get_status_code_retries(service_type))
+        kwargs.setdefault('statsd_prefix', self.get_statsd_prefix())
+        kwargs.setdefault('statsd_client', self.get_statsd_client())
+        kwargs.setdefault('prometheus_counter', self.get_prometheus_counter())
+        kwargs.setdefault(
+            'prometheus_histogram', self.get_prometheus_histogram())
+        kwargs.setdefault('influxdb_config', self._influxdb_config)
+        kwargs.setdefault('influxdb_client', self.get_influxdb_client())
         endpoint_override = self.get_endpoint(service_type)
         version = version_request.version
-        min_api_version = version_request.min_api_version
-        max_api_version = version_request.max_api_version
+        min_api_version = (
+            kwargs.pop('min_version', None) or version_request.min_api_version)
+        max_api_version = (
+            kwargs.pop('max_version', None) or version_request.max_api_version)
+
         # Older neutron has inaccessible discovery document. Nobody noticed
         # because neutronclient hard-codes an append of v2.0. YAY!
-        if service_type == 'network':
+        # Also, older octavia has a similar issue.
+        if service_type in ('network', 'load-balancer'):
             version = None
             min_api_version = None
             max_api_version = None
             if endpoint_override is None:
-                network_adapter = constructor(
-                    session=self.get_session(),
-                    service_type=self.get_service_type(service_type),
-                    service_name=self.get_service_name(service_type),
-                    interface=self.get_interface(service_type),
-                    region_name=self.region_name,
-                )
-                network_endpoint = network_adapter.get_endpoint()
-                if not network_endpoint.rstrip().rsplit('/')[1] == 'v2.0':
-                    if not network_endpoint.endswith('/'):
-                        network_endpoint += '/'
-                    network_endpoint = urllib.parse.urljoin(
-                        network_endpoint, 'v2.0')
-                endpoint_override = network_endpoint
+                endpoint_override = self._get_hardcoded_endpoint(
+                    service_type, constructor)
 
         client = constructor(
             session=self.get_session(),
             service_type=self.get_service_type(service_type),
             service_name=self.get_service_name(service_type),
             interface=self.get_interface(service_type),
-            region_name=self.region_name,
             version=version,
             min_version=min_api_version,
             max_version=max_api_version,
             endpoint_override=endpoint_override,
             default_microversion=version_request.default_microversion,
+            rate_limit=self.get_rate_limit(service_type),
+            concurrency=self.get_concurrency(service_type),
             **kwargs)
         if version_request.default_microversion:
             default_microversion = version_request.default_microversion
@@ -522,6 +815,7 @@ class CloudRegion(object):
         if override_endpoint:
             return override_endpoint
 
+        region_name = self.get_region_name(service_type)
         service_name = self.get_service_name(service_type)
         interface = self.get_interface(service_type)
         session = self.get_session()
@@ -536,7 +830,7 @@ class CloudRegion(object):
             # the request
             endpoint = session.get_endpoint(
                 service_type=service_type,
-                region_name=self.region_name,
+                region_name=region_name,
                 interface=interface,
                 service_name=service_name,
                 **version_kwargs
@@ -546,12 +840,12 @@ class CloudRegion(object):
         if not endpoint:
             self.log.warning(
                 "Keystone catalog entry not found ("
-                "service_type=%s,service_name=%s"
+                "service_type=%s,service_name=%s,"
                 "interface=%s,region_name=%s)",
                 service_type,
                 service_name,
                 interface,
-                self.region_name,
+                region_name,
             )
         return endpoint
 
@@ -689,3 +983,118 @@ class CloudRegion(object):
 
     def get_password_callback(self):
         return self._password_callback
+
+    def get_rate_limit(self, service_type=None):
+        return self._get_service_config(
+            'rate_limit', service_type=service_type)
+
+    def get_concurrency(self, service_type=None):
+        return self._get_service_config(
+            'concurrency', service_type=service_type)
+
+    def get_statsd_client(self):
+        if not statsd:
+            return None
+        statsd_args = {}
+        if self._statsd_host:
+            statsd_args['host'] = self._statsd_host
+        if self._statsd_port:
+            statsd_args['port'] = self._statsd_port
+        if statsd_args:
+            try:
+                return statsd.StatsClient(**statsd_args)
+            except Exception:
+                self.log.warning('Cannot establish connection to statsd')
+                return None
+        else:
+            return None
+
+    def get_statsd_prefix(self):
+        return self._statsd_prefix or 'openstack.api'
+
+    def get_prometheus_registry(self):
+        if not self._collector_registry and prometheus_client:
+            self._collector_registry = prometheus_client.REGISTRY
+        return self._collector_registry
+
+    def get_prometheus_histogram(self):
+        registry = self.get_prometheus_registry()
+        if not registry or not prometheus_client:
+            return
+        # We have to hide a reference to the histogram on the registry
+        # object, because it's collectors must be singletons for a given
+        # registry but register at creation time.
+        hist = getattr(registry, '_openstacksdk_histogram', None)
+        if not hist:
+            hist = prometheus_client.Histogram(
+                'openstack_http_response_time',
+                'Time taken for an http response to an OpenStack service',
+                labelnames=[
+                    'method', 'endpoint', 'service_type', 'status_code'
+                ],
+                registry=registry,
+            )
+            registry._openstacksdk_histogram = hist
+        return hist
+
+    def get_prometheus_counter(self):
+        registry = self.get_prometheus_registry()
+        if not registry or not prometheus_client:
+            return
+        counter = getattr(registry, '_openstacksdk_counter', None)
+        if not counter:
+            counter = prometheus_client.Counter(
+                'openstack_http_requests',
+                'Number of HTTP requests made to an OpenStack service',
+                labelnames=[
+                    'method', 'endpoint', 'service_type', 'status_code'
+                ],
+                registry=registry,
+            )
+            registry._openstacksdk_counter = counter
+        return counter
+
+    def has_service(self, service_type):
+        service_type = service_type.lower().replace('-', '_')
+        key = 'has_{service_type}'.format(service_type=service_type)
+        return self.config.get(
+            key, self._service_type_manager.is_official(service_type))
+
+    def disable_service(self, service_type, reason=None):
+        _disable_service(self.config, service_type, reason=reason)
+
+    def enable_service(self, service_type):
+        service_type = service_type.lower().replace('-', '_')
+        key = 'has_{service_type}'.format(service_type=service_type)
+        self.config[key] = True
+
+    def get_disabled_reason(self, service_type):
+        service_type = service_type.lower().replace('-', '_')
+        d_key = _make_key('disabled_reason', service_type)
+        return self.config.get(d_key)
+
+    def get_influxdb_client(self):
+        influx_args = {}
+        if not self._influxdb_config:
+            return None
+        use_udp = bool(self._influxdb_config.get('use_udp', False))
+        port = self._influxdb_config.get('port')
+        if use_udp:
+            influx_args['use_udp'] = True
+        if 'port' in self._influxdb_config:
+            if use_udp:
+                influx_args['udp_port'] = port
+            else:
+                influx_args['port'] = port
+        for key in ['host', 'username', 'password', 'database', 'timeout']:
+            if key in self._influxdb_config:
+                influx_args[key] = self._influxdb_config[key]
+        if influxdb and influx_args:
+            try:
+                return influxdb.InfluxDBClient(**influx_args)
+            except Exception:
+                self.log.warning('Cannot establish connection to InfluxDB')
+        else:
+            self.log.warning('InfluxDB configuration is present, '
+                             'but no client library is found.')
+        return None
