@@ -47,7 +47,7 @@ is the recommended way to go:
 If the application in question is a command line application that should also
 accept command line arguments, an `argparse.Namespace` can be passed to
 :func:`openstack.connect` that will have relevant arguments added to it and
-then subsequently consumed by the construtor:
+then subsequently consumed by the constructor:
 
 .. code-block:: python
 
@@ -112,6 +112,27 @@ is needed:
         compute_api_version='2',
         identity_interface='internal')
 
+From oslo.conf CONF object
+--------------------------
+
+For applications that have an oslo.config ``CONF`` object that has been
+populated with ``keystoneauth1.loading.register_adapter_conf_options`` in
+groups named by the OpenStack service's project name, it is possible to
+construct a Connection with the ``CONF`` object and an authenticated Session.
+
+.. note::
+
+    This is primarily intended for use by OpenStack services to talk amongst
+    themselves.
+
+.. code-block:: python
+
+    from openstack import connection
+
+    conn = connection.Connection(
+        session=session,
+        oslo_conf=CONF)
+
 From existing CloudRegion
 -------------------------
 
@@ -156,19 +177,41 @@ Additional information about the services can be found in the
 :ref:`service-proxies` documentation.
 """
 import warnings
+import weakref
 
+try:
+    # For python 3.8 and later
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    # For everyone else
+    import importlib_metadata
+
+import concurrent.futures
 import keystoneauth1.exceptions
 import requestsexceptions
-import six
 
 from openstack import _log
-from openstack._meta import connection as _meta
-from openstack import cloud as _cloud
+from openstack import _services_mixin
+from openstack.cloud import openstackcloud as _cloud
+from openstack.cloud import _accelerator
+from openstack.cloud import _baremetal
+from openstack.cloud import _block_storage
+from openstack.cloud import _compute
+from openstack.cloud import _clustering
+from openstack.cloud import _coe
+from openstack.cloud import _dns
+from openstack.cloud import _floating_ip
+from openstack.cloud import _identity
+from openstack.cloud import _image
+from openstack.cloud import _network
+from openstack.cloud import _network_common
+from openstack.cloud import _object_store
+from openstack.cloud import _orchestration
+from openstack.cloud import _security_group
 from openstack import config as _config
 from openstack.config import cloud_region
 from openstack import exceptions
 from openstack import service_description
-from openstack import task_manager as _task_manager
 
 __all__ = [
     'from_config',
@@ -195,7 +238,7 @@ def from_config(cloud=None, config=None, options=None, **kwargs):
     :param argparse.Namespace options:
         Allows direct passing in of options to be added to the cloud config.
         This does not have to be an actual instance of argparse.Namespace,
-        despite the naming of the the
+        despite the naming of the
         `openstack.config.loader.OpenStackConfig.get_one` argument to which
         it is passed.
 
@@ -211,18 +254,38 @@ def from_config(cloud=None, config=None, options=None, **kwargs):
     return Connection(config=config)
 
 
-class Connection(six.with_metaclass(_meta.ConnectionMeta,
-                                    _cloud.OpenStackCloud)):
+class Connection(
+    _services_mixin.ServicesMixin,
+    _cloud._OpenStackCloudMixin,
+    _accelerator.AcceleratorCloudMixin,
+    _baremetal.BaremetalCloudMixin,
+    _block_storage.BlockStorageCloudMixin,
+    _compute.ComputeCloudMixin,
+    _clustering.ClusteringCloudMixin,
+    _coe.CoeCloudMixin,
+    _dns.DnsCloudMixin,
+    _floating_ip.FloatingIPCloudMixin,
+    _identity.IdentityCloudMixin,
+    _image.ImageCloudMixin,
+    _network.NetworkCloudMixin,
+    _network_common.NetworkCommonCloudMixin,
+    _object_store.ObjectStoreCloudMixin,
+    _orchestration.OrchestrationCloudMixin,
+    _security_group.SecurityGroupCloudMixin
+):
 
     def __init__(self, cloud=None, config=None, session=None,
                  app_name=None, app_version=None,
-                 # TODO(shade) Remove these once we've shifted
-                 # python-openstackclient to not use the profile interface.
-                 authenticator=None, profile=None,
                  extra_services=None,
                  strict=False,
                  use_direct_get=False,
                  task_manager=None,
+                 rate_limit=None,
+                 oslo_conf=None,
+                 service_types=None,
+                 global_request_id=None,
+                 strict_proxies=False,
+                 pool_executor=None,
                  **kwargs):
         """Create a connection to a cloud.
 
@@ -252,14 +315,6 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         :param str app_name: Name of the application to be added to User Agent.
         :param str app_version: Version of the application to be added to
             User Agent.
-        :param authenticator: DEPRECATED. Only exists for short-term backwards
-                              compatibility for python-openstackclient while we
-                              transition. See :doc:`transition_from_profile`
-                              for details.
-        :param profile: DEPRECATED. Only exists for short-term backwards
-                        compatibility for python-openstackclient while we
-                        transition. See :doc:`transition_from_profile`
-                        for details.
         :param extra_services: List of
             :class:`~openstack.service_description.ServiceDescription`
             objects describing services that openstacksdk otherwise does not
@@ -269,33 +324,63 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
             filtering instead of making list calls and filtering client-side.
             Default false.
         :param task_manager:
-            Task Manager to handle the execution of remote REST calls.
-            Defaults to None which causes a direct-action Task Manager to be
-            used.
-        :type manager: :class:`~openstack.task_manager.TaskManager`
+            Ignored. Exists for backwards compat during transition. Rate limit
+            parameters should be passed directly to the `rate_limit` parameter.
+        :param rate_limit:
+            Client-side rate limit, expressed in calls per second. The
+            parameter can either be a single float, or it can be a dict with
+            keys as service-type and values as floats expressing the calls
+            per second for that service. Defaults to None, which means no
+            rate-limiting is performed.
+        :param oslo_conf: An oslo.config CONF object.
+        :type oslo_conf: :class:`~oslo_config.cfg.ConfigOpts`
+            An oslo.config ``CONF`` object that has been populated with
+            ``keystoneauth1.loading.register_adapter_conf_options`` in
+            groups named by the OpenStack service's project name.
+        :param service_types:
+            A list/set of service types this Connection should support. All
+            other service types will be disabled (will error if used).
+            **Currently only supported in conjunction with the ``oslo_conf``
+            kwarg.**
+        :param global_request_id: A Request-id to send with all interactions.
+        :param strict_proxies:
+            If True, check proxies on creation and raise
+            ServiceDiscoveryException if the service is unavailable.
+        :type strict_proxies: bool
+            Throw an ``openstack.exceptions.ServiceDiscoveryException`` if the
+            endpoint for a given service doesn't work. This is useful for
+            OpenStack services using sdk to talk to other OpenStack services
+            where it can be expected that the deployer config is correct and
+            errors should be reported immediately.
+            Default false.
+        :param pool_executor:
+        :type pool_executor: :class:`~futurist.Executor`
+            A futurist ``Executor`` object to be used for concurrent background
+            activities. Defaults to None in which case a ThreadPoolExecutor
+            will be created if needed.
         :param kwargs: If a config is not provided, the rest of the parameters
             provided are assumed to be arguments to be passed to the
-            CloudRegion contructor.
+            CloudRegion constructor.
         """
         self.config = config
         self._extra_services = {}
+        self._strict_proxies = strict_proxies
         if extra_services:
             for service in extra_services:
                 self._extra_services[service.service_type] = service
 
         if not self.config:
-            if profile:
-                import openstack.profile
-                # TODO(shade) Remove this once we've shifted
-                # python-openstackclient to not use the profile interface.
-                self.config = openstack.profile._get_config_from_profile(
-                    profile, authenticator, **kwargs)
+            if oslo_conf:
+                self.config = cloud_region.from_conf(
+                    oslo_conf, session=session, app_name=app_name,
+                    app_version=app_version, service_types=service_types)
             elif session:
                 self.config = cloud_region.from_session(
                     session=session,
                     app_name=app_name, app_version=app_version,
                     load_yaml_config=False,
                     load_envvars=False,
+                    rate_limit=rate_limit,
                     **kwargs)
             else:
                 self.config = _config.get_cloud_region(
@@ -303,18 +388,69 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
                     app_name=app_name, app_version=app_version,
                     load_yaml_config=cloud is not None,
                     load_envvars=cloud is not None,
+                    rate_limit=rate_limit,
                     **kwargs)
 
-        self.task_manager = task_manager or _task_manager.TaskManager(
-            self.config.full_name)
-
         self._session = None
         self._proxies = {}
+        self.__pool_executor = pool_executor
+        self._global_request_id = global_request_id
         self.use_direct_get = use_direct_get
         self.strict_mode = strict
-        # Call the OpenStackCloud constructor while we work on integrating
-        # things better.
-        _cloud.OpenStackCloud.__init__(self)
+        # Call the _*CloudMixin constructors while we work on
+        # integrating things better.
+        _cloud._OpenStackCloudMixin.__init__(self)
+        _accelerator.AcceleratorCloudMixin.__init__(self)
+        _baremetal.BaremetalCloudMixin.__init__(self)
+        _block_storage.BlockStorageCloudMixin.__init__(self)
+        _clustering.ClusteringCloudMixin.__init__(self)
+        _coe.CoeCloudMixin.__init__(self)
+        _compute.ComputeCloudMixin.__init__(self)
+        _dns.DnsCloudMixin.__init__(self)
+        _floating_ip.FloatingIPCloudMixin.__init__(self)
+        _identity.IdentityCloudMixin.__init__(self)
+        _image.ImageCloudMixin.__init__(self)
+        _network_common.NetworkCommonCloudMixin.__init__(self)
+        _network.NetworkCloudMixin.__init__(self)
+        _object_store.ObjectStoreCloudMixin.__init__(self)
+        _orchestration.OrchestrationCloudMixin.__init__(self)
+        _security_group.SecurityGroupCloudMixin.__init__(self)
+
+        # Allow vendors to provide hooks. They will normally only receive a
+        # connection object and a responsible to register additional services
+        vendor_hook = kwargs.get('vendor_hook')
+        if not vendor_hook and 'vendor_hook' in self.config.config:
+            # Get the one from profile
+            vendor_hook = self.config.config.get('vendor_hook')
+        if vendor_hook:
+            try:
+                # NOTE(gtema): no class name in the hook, plain module:function
+                # Split string hook into module and function
+                try:
+                    (package_name, function) = vendor_hook.rsplit(':')
+
+                    if package_name and function:
+                        ep = importlib_metadata.EntryPoint(
+                            name='vendor_hook',
+                            value=vendor_hook,
+                            group='vendor_hook',
+                        )
+                        hook = ep.load()
+                        hook(self)
+                except ValueError:
+                    self.log.warning('Hook should be in the entrypoint '
+                                     'module:attribute format')
+            except (ImportError, TypeError, AttributeError) as e:
+                self.log.warning('Configured hook %s cannot be executed: %s',
+                                 vendor_hook, e)
+
+        # Add additional metrics into the configuration according to the
+        # selected connection. We don't want to deal with overall config in the
+        # proxy, just pass required part.
+        if (self.config._influxdb_config
+                and 'additional_metric_tags' in self.config.config):
+            self.config._influxdb_config['additional_metric_tags'] = \
+                self.config.config['additional_metric_tags']
 
     @property
     def session(self):
@@ -323,7 +459,7 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
             # Hide a reference to the connection on the session to help with
             # backwards compatibility for folks trying to just pass
             # conn.session to a Resource method's session argument.
-            self.session._sdk_connection = self
+            self.session._sdk_connection = weakref.proxy(self)
         return self._session
 
     def add_service(self, service):
@@ -345,12 +481,22 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         """
         # If we don't have a proxy, just instantiate Proxy so that
         # we get an adapter.
-        if isinstance(service, six.string_types):
+        if isinstance(service, str):
             service = service_description.ServiceDescription(service)
 
-        # Register the proxy class with every known alias
+        # Directly invoke descriptor of the ServiceDescription
+        def getter(self):
+            return service.__get__(self, service)
+
+        # Register the ServiceDescription class (as property)
+        # with every known alias for a "runtime descriptor"
         for attr_name in service.all_types:
-            setattr(self, attr_name.replace('-', '_'), service)
+            setattr(
+                self.__class__,
+                attr_name.replace('-', '_'),
+                property(fget=getter)
+            )
+        self.config.enable_service(service.service_type)
 
     def authorize(self):
         """Authorize this Connection
@@ -369,4 +515,25 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         try:
             return self.session.get_token()
         except keystoneauth1.exceptions.ClientException as e:
-            raise exceptions.raise_from_response(e.response)
+            raise exceptions.SDKException(e)
+
+    @property
+    def _pool_executor(self):
+        if not self.__pool_executor:
+            self.__pool_executor = concurrent.futures.ThreadPoolExecutor(
+                max_workers=5)
+        return self.__pool_executor
+
+    def close(self):
+        """Release any resources held open."""
+        if self.__pool_executor:
+            self.__pool_executor.shutdown()
+
+    def set_global_request_id(self, global_request_id):
+        self._global_request_id = global_request_id
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
