@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.compute import version
 
@@ -23,22 +23,21 @@ EXAMPLE = {
 }
 
 
-class TestVersion(testtools.TestCase):
+class TestVersion(base.TestCase):
 
     def test_basic(self):
         sot = version.Version()
         self.assertEqual('version', sot.resource_key)
         self.assertEqual('versions', sot.resources_key)
         self.assertEqual('/', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_retrieve)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
     def test_make_it(self):
-        sot = version.Version(EXAMPLE)
+        sot = version.Version(**EXAMPLE)
         self.assertEqual(EXAMPLE['id'], sot.id)
         self.assertEqual(EXAMPLE['links'], sot.links)
         self.assertEqual(EXAMPLE['status'], sot.status)
