@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.compute.v2 import flavor
 
@@ -19,6 +19,7 @@ BASIC_EXAMPLE = {
     'id': IDENTIFIER,
     'links': '2',
     'name': '3',
+    'description': 'Testing flavor',
     'disk': 4,
     'os-flavor-access:is_public': True,
     'ram': 6,
@@ -30,24 +31,26 @@ BASIC_EXAMPLE = {
 }
 
 
-class TestFlavor(testtools.TestCase):
+class TestFlavor(base.TestCase):
 
     def test_basic(self):
         sot = flavor.Flavor()
         self.assertEqual('flavor', sot.resource_key)
         self.assertEqual('flavors', sot.resources_key)
         self.assertEqual('/flavors', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
+        self.assertTrue(sot.allow_fetch)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_commit)
 
         self.assertDictEqual({"sort_key": "sort_key",
                               "sort_dir": "sort_dir",
                               "min_disk": "minDisk",
-                              "min_ram": "minRam"},
+                              "min_ram": "minRam",
+                              "limit": "limit",
+                              "marker": "marker",
+                              "is_public": "is_public"},
                              sot._query_mapping._mapping)
 
     def test_make_basic(self):
@@ -55,6 +58,7 @@ class TestFlavor(testtools.TestCase):
         self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
         self.assertEqual(BASIC_EXAMPLE['links'], sot.links)
         self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
+        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
         self.assertEqual(BASIC_EXAMPLE['disk'], sot.disk)
         self.assertEqual(BASIC_EXAMPLE['os-flavor-access:is_public'],
                          sot.is_public)
@@ -72,9 +76,8 @@ class TestFlavor(testtools.TestCase):
         self.assertEqual('flavor', sot.resource_key)
         self.assertEqual('flavors', sot.resources_key)
         self.assertEqual('/flavors/detail', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_get)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertTrue(sot.allow_list)
