@@ -11,12 +11,12 @@
 # under the License.
 
 import copy
+from unittest import mock
 
 from keystoneauth1 import adapter
-import mock
-from openstack.tests.unit import base
 
 from openstack.compute.v2 import limits
+from openstack.tests.unit import base
 
 ABSOLUTE_LIMITS = {
     "maxImageMeta": 128,
@@ -69,10 +69,9 @@ class TestAbsoluteLimits(base.TestCase):
         self.assertIsNone(sot.resource_key)
         self.assertIsNone(sot.resources_key)
         self.assertEqual("", sot.base_path)
-        self.assertIsNone(sot.service)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_get)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertFalse(sot.allow_list)
 
@@ -121,8 +120,8 @@ class TestRateLimit(base.TestCase):
         self.assertEqual("", sot.base_path)
         self.assertIsNone(sot.service)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_get)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertFalse(sot.allow_list)
 
@@ -139,22 +138,22 @@ class TestLimits(base.TestCase):
         sot = limits.Limits()
         self.assertEqual("limits", sot.resource_key)
         self.assertEqual("/limits", sot.base_path)
-        self.assertEqual("compute", sot.service.service_type)
-        self.assertTrue(sot.allow_get)
+        self.assertTrue(sot.allow_fetch)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertFalse(sot.allow_list)
 
     def test_get(self):
         sess = mock.Mock(spec=adapter.Adapter)
+        sess.default_microversion = None
         resp = mock.Mock()
         sess.get.return_value = resp
         resp.json.return_value = copy.deepcopy(LIMITS_BODY)
         resp.headers = {}
         resp.status_code = 200
 
-        sot = limits.Limits().get(sess)
+        sot = limits.Limits().fetch(sess)
 
         self.assertEqual(ABSOLUTE_LIMITS["maxImageMeta"],
                          sot.absolute.image_meta)
