@@ -10,51 +10,109 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
-import six
-from openstack.tests.unit import base
+from unittest import mock
 
 from openstack.compute.v2 import server
+from openstack.image.v2 import image
+from openstack.tests.unit import base
 
 IDENTIFIER = 'IDENTIFIER'
 EXAMPLE = {
-    'accessIPv4': '1',
-    'accessIPv6': '2',
-    'addresses': {'region': '3'},
-    'config_drive': True,
-    'created': '2015-03-09T12:14:57.233772',
+    'OS-DCF:diskConfig': 'AUTO',
+    'OS-EXT-AZ:availability_zone': 'us-west',
+    'OS-EXT-SRV-ATTR:host': 'compute',
+    'OS-EXT-SRV-ATTR:hostname': 'new-server-test',
+    'OS-EXT-SRV-ATTR:hypervisor_hostname': 'fake-mini',
+    'OS-EXT-SRV-ATTR:instance_name': 'instance-00000001',
+    'OS-EXT-SRV-ATTR:kernel_id': '',
+    'OS-EXT-SRV-ATTR:launch_index': 0,
+    'OS-EXT-SRV-ATTR:ramdisk_id': '',
+    'OS-EXT-SRV-ATTR:reservation_id': 'r-ov3q80zj',
+    'OS-EXT-SRV-ATTR:root_device_name': '/dev/sda',
+    'OS-EXT-SRV-ATTR:user_data': 'IyEvYmluL2Jhc2gKL2Jpbi9IHlvdSEiCg==',
+    'OS-EXT-STS:power_state': 1,
+    'OS-EXT-STS:task_state': None,
+    'OS-EXT-STS:vm_state': 'active',
+    'OS-SRV-USG:launched_at': '2017-02-14T19:23:59.895661',
+    'OS-SRV-USG:terminated_at': '2015-03-09T12:15:57.233772',
+    'OS-SCH-HNT:scheduler_hints': {'key': '30'},
+    'accessIPv4': '1.2.3.4',
+    'accessIPv6': '80fe::',
+    'adminPass': '27',
+    'addresses': {
+        'private': [
+            {
+                'OS-EXT-IPS-MAC:mac_addr': 'aa:bb:cc:dd:ee:ff',
+                'OS-EXT-IPS:type': 'fixed',
+                'addr': '192.168.0.3',
+                'version': 4
+            }
+        ]
+    },
+    'block_device_mapping_v2': {'key': '29'},
+    'config_drive': '',
+    'created': '2017-02-14T19:23:58Z',
+    'description': 'dummy',
     'flavorRef': '5',
-    'flavor': {'id': 'FLAVOR_ID', 'links': {}},
-    'hostId': '6',
+    'flavor': {
+        'disk': 1,
+        'ephemeral': 0,
+        'extra_specs': {
+            'hw:cpu_policy': 'dedicated',
+            'hw:mem_page_size': '2048'
+        },
+        'original_name': 'm1.tiny.specs',
+        'ram': 512,
+        'swap': 0,
+        'vcpus': 1
+    },
+    'hostId': '2091634baaccdc4c5a1d57069c833e402921df696b7f970791b12ec6',
+    'host_status': 'UP',
     'id': IDENTIFIER,
     'imageRef': '8',
-    'image': {'id': 'IMAGE_ID', 'links': {}},
-    'links': '9',
-    'metadata': {'key': '10'},
+    'image': {
+        'id': '70a599e0-31e7-49b7-b260-868f441e862b',
+        'links': [
+            {
+                'href': 'http://openstack.example.com/images/70a599e0',
+                'rel': 'bookmark'
+            }
+        ]
+    },
+    'key_name': 'dummy',
+    'links': [
+        {
+            'href': 'http://openstack.example.com/v2.1/servers/9168b536',
+            'rel': 'self'
+        },
+        {
+            'href': 'http://openstack.example.com/servers/9168b536',
+            'rel': 'bookmark'
+        }
+    ],
+    'locked': True,
+    'metadata': {
+        'My Server Name': 'Apache1'
+    },
+    'name': 'new-server-test',
     'networks': 'auto',
-    'name': '11',
-    'progress': 12,
-    'tenant_id': '13',
-    'status': '14',
-    'updated': '2015-03-09T12:15:57.233772',
-    'user_id': '16',
-    'key_name': '17',
-    'OS-DCF:diskConfig': '18',
-    'OS-EXT-AZ:availability_zone': '19',
-    'OS-EXT-STS:power_state': '20',
-    'OS-EXT-STS:task_state': '21',
-    'OS-EXT-STS:vm_state': '22',
-    'os-extended-volumes:volumes_attached': '23',
-    'OS-SRV-USG:launched_at': '2015-03-09T12:15:57.233772',
-    'OS-SRV-USG:terminated_at': '2015-03-09T12:15:57.233772',
-    'security_groups': '26',
-    'adminPass': '27',
+    'os-extended-volumes:volumes_attached': [],
     'personality': '28',
-    'block_device_mapping_v2': {'key': '29'},
-    'OS-EXT-SRV-ATTR:hypervisor_hostname': 'hypervisor.example.com',
-    'OS-EXT-SRV-ATTR:instance_name': 'instance-00000001',
-    'OS-SCH-HNT:scheduler_hints': {'key': '30'},
-    'OS-EXT-SRV-ATTR:user_data': '31'
+    'progress': 0,
+    'security_groups': [
+        {
+            'name': 'default'
+        }
+    ],
+    'status': 'ACTIVE',
+    'tags': [],
+    'tenant_id': '6f70656e737461636b20342065766572',
+    'trusted_image_certificates': [
+        '0b5d2c72-12cc-4ba6-a8d7-3ff5cc1d8cb8',
+        '674736e3-f25c-405c-8362-bbf991e0ce0a'
+    ],
+    'updated': '2017-02-14T19:24:00Z',
+    'user_id': 'fake'
 }
 
 
@@ -65,6 +123,7 @@ class TestServer(base.TestCase):
         self.resp = mock.Mock()
         self.resp.body = None
         self.resp.json = mock.Mock(return_value=self.resp.body)
+        self.resp.status_code = 200
         self.sess = mock.Mock()
         self.sess.post = mock.Mock(return_value=self.resp)
 
@@ -73,33 +132,57 @@ class TestServer(base.TestCase):
         self.assertEqual('server', sot.resource_key)
         self.assertEqual('servers', sot.resources_key)
         self.assertEqual('/servers', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
-        self.assertDictEqual({"image": "image",
-                              "flavor": "flavor",
-                              "name": "name",
-                              "status": "status",
-                              "host": "host",
-                              "all_tenants": "all_tenants",
+        self.assertDictEqual({"access_ipv4": "access_ip_v4",
+                              "access_ipv6": "access_ip_v6",
+                              "auto_disk_config": "auto_disk_config",
+                              "availability_zone": "availability_zone",
+                              "changes_before": "changes-before",
                               "changes_since": "changes-since",
-                              "limit": "limit",
-                              "marker": "marker",
-                              "sort_key": "sort_key",
-                              "sort_dir": "sort_dir",
-                              "reservation_id": "reservation_id",
-                              "project_id": "project_id",
-                              "tags": "tags",
-                              "tags_any": "tags-any",
-                              "not_tags": "not-tags",
-                              "not_tags_any": "not-tags-any",
-                              "is_deleted": "deleted",
+                              "compute_host": "host",
+                              "has_config_drive": "config_drive",
+                              "created_at": "created_at",
+                              "description": "description",
+                              "flavor": "flavor",
+                              "hostname": "hostname",
+                              "image": "image",
                               "ipv4_address": "ip",
                               "ipv6_address": "ip6",
+                              "id": "uuid",
+                              "deleted_only": "deleted",
+                              "is_soft_deleted": "soft_deleted",
+                              "kernel_id": "kernel_id",
+                              "key_name": "key_name",
+                              "launch_index": "launch_index",
+                              "launched_at": "launched_at",
+                              "limit": "limit",
+                              "locked_by": "locked_by",
+                              "marker": "marker",
+                              "name": "name",
+                              "node": "node",
+                              "power_state": "power_state",
+                              "progress": "progress",
+                              "project_id": "project_id",
+                              "ramdisk_id": "ramdisk_id",
+                              "reservation_id": "reservation_id",
+                              "root_device_name": "root_device_name",
+                              "sort_dir": "sort_dir",
+                              "sort_key": "sort_key",
+                              "status": "status",
+                              "task_state": "task_state",
+                              "terminated_at": "terminated_at",
+                              "user_id": "user_id",
+                              "vm_state": "vm_state",
+                              "all_projects": "all_tenants",
+                              "tags": "tags",
+                              "any_tags": "tags-any",
+                              "not_tags": "not-tags",
+                              "not_any_tags": "not-tags-any",
                               },
                              sot._query_mapping._mapping)
 
@@ -113,9 +196,10 @@ class TestServer(base.TestCase):
         self.assertEqual(EXAMPLE['flavorRef'], sot.flavor_id)
         self.assertEqual(EXAMPLE['flavor'], sot.flavor)
         self.assertEqual(EXAMPLE['hostId'], sot.host_id)
+        self.assertEqual(EXAMPLE['host_status'], sot.host_status)
         self.assertEqual(EXAMPLE['id'], sot.id)
         self.assertEqual(EXAMPLE['imageRef'], sot.image_id)
-        self.assertEqual(EXAMPLE['image'], sot.image)
+        self.assertEqual(image.Image(**EXAMPLE['image']), sot.image)
         self.assertEqual(EXAMPLE['links'], sot.links)
         self.assertEqual(EXAMPLE['metadata'], sot.metadata)
         self.assertEqual(EXAMPLE['networks'], sot.networks)
@@ -142,25 +226,30 @@ class TestServer(base.TestCase):
         self.assertEqual(EXAMPLE['personality'], sot.personality)
         self.assertEqual(EXAMPLE['block_device_mapping_v2'],
                          sot.block_device_mapping)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:host'],
+                         sot.compute_host)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:hostname'],
+                         sot.hostname)
         self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:hypervisor_hostname'],
                          sot.hypervisor_hostname)
         self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:instance_name'],
                          sot.instance_name)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:kernel_id'],
+                         sot.kernel_id)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:launch_index'],
+                         sot.launch_index)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:ramdisk_id'],
+                         sot.ramdisk_id)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:reservation_id'],
+                         sot.reservation_id)
+        self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:root_device_name'],
+                         sot.root_device_name)
         self.assertEqual(EXAMPLE['OS-SCH-HNT:scheduler_hints'],
                          sot.scheduler_hints)
         self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:user_data'], sot.user_data)
-
-    def test_detail(self):
-        sot = server.ServerDetail()
-        self.assertEqual('server', sot.resource_key)
-        self.assertEqual('servers', sot.resources_key)
-        self.assertEqual('/servers/detail', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
-        self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_get)
-        self.assertFalse(sot.allow_update)
-        self.assertFalse(sot.allow_delete)
-        self.assertTrue(sot.allow_list)
+        self.assertEqual(EXAMPLE['locked'], sot.is_locked)
+        self.assertEqual(EXAMPLE['trusted_image_certificates'],
+                         sot.trusted_image_certificates)
 
     def test__prepare_server(self):
         zone = 1
@@ -306,30 +395,88 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion=None)
 
-    def test_create_image(self):
+    def test_create_image_header(self):
         sot = server.Server(**EXAMPLE)
         name = 'noo'
         metadata = {'nu': 'image', 'created': 'today'}
 
-        self.assertIsNone(sot.create_image(self.sess, name, metadata))
+        url = 'servers/IDENTIFIER/action'
+        body = {"createImage": {'name': name, 'metadata': metadata}}
+        headers = {'Accept': ''}
+
+        rsp = mock.Mock()
+        rsp.json.return_value = None
+        rsp.headers = {'Location': 'dummy/dummy2'}
+        rsp.status_code = 200
+
+        self.sess.post.return_value = rsp
+
+        self.endpoint_data = mock.Mock(spec=['min_microversion',
+                                             'max_microversion'],
+                                       min_microversion=None,
+                                       max_microversion='2.44')
+        self.sess.get_endpoint_data.return_value = self.endpoint_data
+
+        image_id = sot.create_image(self.sess, name, metadata)
+
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion=None)
+
+        self.assertEqual('dummy2', image_id)
+
+    def test_create_image_microver(self):
+        sot = server.Server(**EXAMPLE)
+        name = 'noo'
+        metadata = {'nu': 'image', 'created': 'today'}
 
         url = 'servers/IDENTIFIER/action'
         body = {"createImage": {'name': name, 'metadata': metadata}}
         headers = {'Accept': ''}
+
+        rsp = mock.Mock()
+        rsp.json.return_value = {'image_id': 'dummy3'}
+        rsp.headers = {'Location': 'dummy/dummy2'}
+        rsp.status_code = 200
+
+        self.sess.post.return_value = rsp
+
+        self.endpoint_data = mock.Mock(spec=['min_microversion',
+                                             'max_microversion'],
+                                       min_microversion='2.1',
+                                       max_microversion='2.56')
+        self.sess.get_endpoint_data.return_value = self.endpoint_data
+
+        image_id = sot.create_image(self.sess, name, metadata)
+
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers, microversion=None)
+            url, json=body, headers=headers, microversion='2.45')
+
+        self.assertEqual('dummy3', image_id)
 
     def test_create_image_minimal(self):
         sot = server.Server(**EXAMPLE)
         name = 'noo'
-
-        self.assertIsNone(self.resp.body, sot.create_image(self.sess, name))
-
         url = 'servers/IDENTIFIER/action'
         body = {"createImage": {'name': name}}
         headers = {'Accept': ''}
+
+        rsp = mock.Mock()
+        rsp.json.return_value = None
+        rsp.headers = {'Location': 'dummy/dummy2'}
+        rsp.status_code = 200
+
+        self.sess.post.return_value = rsp
+
+        self.endpoint_data = mock.Mock(spec=['min_microversion',
+                                             'max_microversion'],
+                                       min_microversion='2.1',
+                                       max_microversion='2.56')
+        self.sess.get_endpoint_data.return_value = self.endpoint_data
+
+        self.assertIsNone(self.resp.body, sot.create_image(self.sess, name))
+
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers, microversion=None)
+            url, json=body, headers=headers, microversion='2.45')
 
     def test_add_security_group(self):
         sot = server.Server(**EXAMPLE)
@@ -659,7 +806,7 @@ class TestServer(base.TestCase):
     def test_live_migrate_no_force(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = None
             max_microversion = None
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -670,26 +817,27 @@ class TestServer(base.TestCase):
             self.sess, host='HOST2', force=False, block_migration=False)
         self.assertIn(
             "Live migration on this cloud implies 'force'",
-            six.text_type(ex))
+            str(ex))
 
     def test_live_migrate_no_microversion_force_true(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = None
             max_microversion = None
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
 
         res = sot.live_migrate(
-            self.sess, host='HOST2', force=True, block_migration=False)
+            self.sess, host='HOST2', force=True, block_migration=True,
+            disk_over_commit=True)
 
         self.assertIsNone(res)
         url = 'servers/IDENTIFIER/action'
         body = {
             'os-migrateLive': {
                 'host': 'HOST2',
-                'disk_over_commit': False,
-                'block_migration': False
+                'disk_over_commit': True,
+                'block_migration': True
             }
         }
 
@@ -700,7 +848,7 @@ class TestServer(base.TestCase):
     def test_live_migrate_25(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.25'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -724,7 +872,7 @@ class TestServer(base.TestCase):
     def test_live_migrate_25_default_block(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.25'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -748,7 +896,7 @@ class TestServer(base.TestCase):
     def test_live_migrate_30(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.30'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -772,7 +920,7 @@ class TestServer(base.TestCase):
     def test_live_migrate_30_force(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.30'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -793,3 +941,48 @@ class TestServer(base.TestCase):
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion='2.30')
+
+    def test_get_security_groups(self):
+        sot = server.Server(**EXAMPLE)
+
+        response = mock.Mock()
+
+        sgs = [{
+            'description': 'default',
+            'id': 1,
+            'name': 'default',
+            'rules': [
+                {
+                    'direction': 'egress',
+                    'ethertype': 'IPv6',
+                    'id': '3c0e45ff-adaf-4124-b083-bf390e5482ff',
+                    'port_range_max': None,
+                    'port_range_min': None,
+                    'protocol': None,
+                    'remote_group_id': None,
+                    'remote_ip_prefix': None,
+                    'security_group_id': '1',
+                    'project_id': 'e4f50856753b4dc6afee5fa6b9b6c550',
+                    'revision_number': 1,
+                    'tags': ['tag1,tag2'],
+                    'tenant_id': 'e4f50856753b4dc6afee5fa6b9b6c550',
+                    'created_at': '2018-03-19T19:16:56Z',
+                    'updated_at': '2018-03-19T19:16:56Z',
+                    'description': ''
+                }
+            ],
+            'tenant_id': 'e4f50856753b4dc6afee5fa6b9b6c550'
+        }]
+
+        response.status_code = 200
+        response.json.return_value = {
+            'security_groups': sgs
+        }
+        self.sess.get.return_value = response
+
+        sot.fetch_security_groups(self.sess)
+
+        url = 'servers/IDENTIFIER/os-security-groups'
+        self.sess.get.assert_called_with(url)
+
+        self.assertEqual(sot.security_groups, sgs)
