@@ -16,7 +16,6 @@ import argparse
 import copy
 import os
 
-import extras
 import fixtures
 import testtools
 import yaml
@@ -40,13 +39,13 @@ class TestConfig(base.TestCase):
                                    vendor_files=[self.vendor_yaml],
                                    secure_files=[self.no_yaml])
         clouds = c.get_all()
-        # We add one by hand because the regions cloud is going to exist
-        # twice since it has two regions in it
+        # We add two by hand because the regions cloud is going to exist
+        # thrice since it has three regions in it
         user_clouds = [
             cloud for cloud in base.USER_CONF['clouds'].keys()
-        ] + ['_test_cloud_regions']
+        ] + ['_test_cloud_regions', '_test_cloud_regions']
         configured_clouds = [cloud.name for cloud in clouds]
-        self.assertItemsEqual(user_clouds, configured_clouds)
+        self.assertCountEqual(user_clouds, configured_clouds)
 
     def test_get_all_clouds(self):
         # Ensure the alias is in place
@@ -54,13 +53,13 @@ class TestConfig(base.TestCase):
                                    vendor_files=[self.vendor_yaml],
                                    secure_files=[self.no_yaml])
         clouds = c.get_all_clouds()
-        # We add one by hand because the regions cloud is going to exist
-        # twice since it has two regions in it
+        # We add two by hand because the regions cloud is going to exist
+        # thrice since it has three regions in it
         user_clouds = [
             cloud for cloud in base.USER_CONF['clouds'].keys()
-        ] + ['_test_cloud_regions']
+        ] + ['_test_cloud_regions', '_test_cloud_regions']
         configured_clouds = [cloud.name for cloud in clouds]
-        self.assertItemsEqual(user_clouds, configured_clouds)
+        self.assertCountEqual(user_clouds, configured_clouds)
 
     def test_get_one(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -96,6 +95,39 @@ class TestConfig(base.TestCase):
         cc = c.get_one()
         self.assertEqual(cc.name, 'single')
 
+    def test_remote_profile(self):
+        single_conf = base._write_yaml({
+            'clouds': {
+                'remote': {
+                    'profile': 'https://example.com',
+                    'auth': {
+                        'username': 'testuser',
+                        'password': 'testpass',
+                        'project_name': 'testproject',
+                    },
+                    'region_name': 'test-region',
+                }
+            }
+        })
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://example.com/.well-known/openstack/api',
+                 json={
+                     "name": "example",
+                     "profile": {
+                         "auth": {
+                             "auth_url": "https://auth.example.com/v3",
+                         }
+                     }
+                 }),
+        ])
+
+        c = config.OpenStackConfig(config_files=[single_conf])
+        cc = c.get_one(cloud='remote')
+        self.assertEqual(cc.name, 'remote')
+        self.assertEqual(cc.auth['auth_url'], 'https://auth.example.com/v3')
+        self.assertEqual(cc.auth['username'], 'testuser')
+
     def test_get_one_auth_defaults(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml])
         cc = c.get_one(cloud='_test-cloud_', auth={'username': 'user'})
@@ -104,22 +136,6 @@ class TestConfig(base.TestCase):
             defaults._defaults['auth_type'],
             cc.auth_type,
         )
-        self.assertEqual(
-            defaults._defaults['identity_api_version'],
-            cc.identity_api_version,
-        )
-
-    def test_get_one_auth_override_defaults(self):
-        default_options = {'compute_api_version': '4'}
-        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
-                                   override_defaults=default_options)
-        cc = c.get_one(cloud='_test-cloud_', auth={'username': 'user'})
-        self.assertEqual('user', cc.auth['username'])
-        self.assertEqual('4', cc.compute_api_version)
-        self.assertEqual(
-            defaults._defaults['identity_api_version'],
-            cc.identity_api_version,
-        )
 
     def test_get_one_with_config_files(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -151,6 +167,23 @@ class TestConfig(base.TestCase):
         self.assertNotIn('domain-id', cc.auth)
         self.assertNotIn('domain_id', cc)
 
+    def test_get_one_unscoped_identity(self):
+        single_conf = base._write_yaml({
+            'clouds': {
+                'unscoped': {
+                    'auth': {
+                        'auth_url': 'http://example.com/v2',
+                        'username': 'testuser',
+                        'password': 'testpass',
+                    },
+                }
+            }
+        })
+        c = config.OpenStackConfig(config_files=[single_conf],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one()
+        self.assertEqual('http://example.com/v2', cc.get_endpoint('identity'))
+
     def test_get_one_domain_scoped(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -158,6 +191,7 @@ class TestConfig(base.TestCase):
         self.assertEqual('12345', cc.auth['domain_id'])
         self.assertNotIn('user_domain_id', cc.auth)
         self.assertNotIn('project_domain_id', cc.auth)
+        self.assertIsNone(cc.get_endpoint('identity'))
 
     def test_get_one_infer_user_domain(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -382,6 +416,13 @@ class TestConfig(base.TestCase):
         self.assertEqual(region, {'name': 'region2', 'values':
                          {'external_network': 'my-network'}})
 
+    def test_get_region_by_name_no_value(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        region = c._get_region(cloud='_test_cloud_regions',
+                               region_name='region-no-value')
+        self.assertEqual(region, {'name': 'region-no-value', 'values': {}})
+
     def test_get_region_invalid_region(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml],
@@ -397,6 +438,31 @@ class TestConfig(base.TestCase):
         region = c._get_region(region_name='no-cloud-region')
         self.assertEqual(region, {'name': 'no-cloud-region', 'values': {}})
 
+    def test_get_region_invalid_keys(self):
+        invalid_conf = base._write_yaml({
+            'clouds': {
+                '_test_cloud': {
+                    'profile': '_test_cloud_in_our_cloud',
+                    'auth': {
+                        'auth_url': 'http://example.com/v2',
+                        'username': 'testuser',
+                        'password': 'testpass',
+                    },
+                    'regions': [
+                        {
+                            'name': 'region1',
+                            'foo': 'bar'
+                        },
+                    ]
+                }
+            }
+        })
+        c = config.OpenStackConfig(config_files=[invalid_conf],
+                                   vendor_files=[self.vendor_yaml])
+        self.assertRaises(
+            exceptions.ConfigException, c._get_region,
+            cloud='_test_cloud', region_name='region1')
+
 
 class TestExcludedFormattedConfigValue(base.TestCase):
     # verify https://storyboard.openstack.org/#!/story/1635696
@@ -698,7 +764,7 @@ class TestConfigArgparse(base.TestCase):
         # Not using assert_cloud_details because of cache settings which
         # are not present without the file
         self.assertIsInstance(cc, cloud_region.CloudRegion)
-        self.assertTrue(extras.safe_hasattr(cc, 'auth'))
+        self.assertTrue(hasattr(cc, 'auth'))
         self.assertIsInstance(cc.auth, dict)
         self.assertIsNone(cc.cloud)
         self.assertIn('username', cc.auth)
