@@ -10,10 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
-import testtools
+from unittest import mock
 
 from openstack.identity.v2 import extension
+from openstack.tests.unit import base
 
 IDENTIFIER = 'IDENTIFIER'
 EXAMPLE = {
@@ -26,22 +26,21 @@ EXAMPLE = {
 }
 
 
-class TestExtension(testtools.TestCase):
+class TestExtension(base.TestCase):
 
     def test_basic(self):
         sot = extension.Extension()
         self.assertEqual('extension', sot.resource_key)
         self.assertEqual('extensions', sot.resources_key)
         self.assertEqual('/extensions', sot.base_path)
-        self.assertEqual('identity', sot.service.service_type)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_retrieve)
-        self.assertFalse(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
     def test_make_it(self):
-        sot = extension.Extension(EXAMPLE)
+        sot = extension.Extension(**EXAMPLE)
         self.assertEqual(EXAMPLE['alias'], sot.alias)
         self.assertEqual(EXAMPLE['description'], sot.description)
         self.assertEqual(EXAMPLE['links'], sot.links)
@@ -62,7 +61,7 @@ class TestExtension(testtools.TestCase):
         resp.json = mock.Mock(return_value=resp.body)
         session = mock.Mock()
         session.get = mock.Mock(return_value=resp)
-        sot = extension.Extension(EXAMPLE)
+        sot = extension.Extension(**EXAMPLE)
         result = sot.list(session)
         self.assertEqual(next(result).name, 'a')
         self.assertEqual(next(result).name, 'b')
