@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.image.v2 import member
 
@@ -24,17 +24,16 @@ EXAMPLE = {
 }
 
 
-class TestMember(testtools.TestCase):
+class TestMember(base.TestCase):
     def test_basic(self):
         sot = member.Member()
         self.assertIsNone(sot.resource_key)
         self.assertEqual('members', sot.resources_key)
         self.assertEqual('/images/%(image_id)s/members', sot.base_path)
-        self.assertEqual('image', sot.service.service_type)
         self.assertEqual('member', sot._alternate_id())
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
