@@ -34,11 +34,21 @@ EXAMPLE = {
     'extra_dhcp_opts': [{'13': 13}],
     'fixed_ips': [{'14': '14'}],
     'id': IDENTIFIER,
+    'ip_allocation': 'immediate',
     'mac_address': '16',
     'name': '17',
     'network_id': '18',
+    'numa_affinity_policy': False,
     'port_security_enabled': True,
+    'qos_network_policy_id': '32',
     'qos_policy_id': '21',
+    'propagate_uplink_status': False,
+    'resource_request': {
+        'required': [
+            'CUSTOM_PHYSNET_PUBLIC', 'CUSTOM_VNIC_TYPE_NORMAL'],
+        'resources': {
+            'NET_BW_EGR_KILOBIT_PER_SEC': 1,
+            'NET_BW_IGR_KILOBIT_PER_SEC': 2}},
     'revision_number': 22,
     'security_groups': ['23'],
     'status': '25',
@@ -61,10 +71,9 @@ class TestPort(base.TestCase):
         self.assertEqual('port', sot.resource_key)
         self.assertEqual('ports', sot.resources_key)
         self.assertEqual('/ports', sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
@@ -77,6 +86,7 @@ class TestPort(base.TestCase):
                               "device_id": "device_id",
                               "device_owner": "device_owner",
                               "fixed_ips": "fixed_ips",
+                              "id": "id",
                               "ip_address": "ip_address",
                               "mac_address": "mac_address",
                               "name": "name",
@@ -117,11 +127,19 @@ class TestPort(base.TestCase):
         self.assertEqual(EXAMPLE['extra_dhcp_opts'], sot.extra_dhcp_opts)
         self.assertEqual(EXAMPLE['fixed_ips'], sot.fixed_ips)
         self.assertEqual(EXAMPLE['id'], sot.id)
+        self.assertEqual(EXAMPLE['ip_allocation'], sot.ip_allocation)
         self.assertEqual(EXAMPLE['mac_address'], sot.mac_address)
         self.assertEqual(EXAMPLE['name'], sot.name)
         self.assertEqual(EXAMPLE['network_id'], sot.network_id)
+        self.assertEqual(EXAMPLE['numa_affinity_policy'],
+                         sot.numa_affinity_policy)
         self.assertTrue(sot.is_port_security_enabled)
+        self.assertEqual(EXAMPLE['qos_network_policy_id'],
+                         sot.qos_network_policy_id)
         self.assertEqual(EXAMPLE['qos_policy_id'], sot.qos_policy_id)
+        self.assertEqual(EXAMPLE['propagate_uplink_status'],
+                         sot.propagate_uplink_status)
+        self.assertEqual(EXAMPLE['resource_request'], sot.resource_request)
         self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
         self.assertEqual(EXAMPLE['security_groups'], sot.security_group_ids)
         self.assertEqual(EXAMPLE['status'], sot.status)
