@@ -61,10 +61,9 @@ class TestObject(base_test_object.BaseTestObject):
         self.assertIsNone(sot.resources_key)
         self.assertEqual('name', sot._alternate_id())
         self.assertEqual('/%(container)s', sot.base_path)
-        self.assertEqual('object-store', sot.service.service_type)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
+        self.assertTrue(sot.allow_fetch)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
         self.assertTrue(sot.allow_head)
@@ -106,7 +105,7 @@ class TestObject(base_test_object.BaseTestObject):
         headers = {
             'X-Newest': 'True',
             'If-Match': self.headers['Etag'],
-            'Accept': 'bytes'
+            'Accept': '*/*'
         }
         self.register_uris([
             dict(method='GET', uri=self.object_endpoint,
@@ -122,7 +121,7 @@ class TestObject(base_test_object.BaseTestObject):
         # the up-conversion works properly.
         sot.if_match = self.headers['Etag']
 
-        rv = sot.download(self.conn.object_store)
+        rv = sot.download(self.cloud.object_store)
 
         self.assertEqual(self.the_data, rv)
 
@@ -132,7 +131,7 @@ class TestObject(base_test_object.BaseTestObject):
         sot = obj.Object.new(container=self.container, name=self.object,
                              data=data)
         sot.is_newest = True
-        sent_headers = {"x-newest": 'True', "Accept": ""}
+        sent_headers = {"x-newest": 'True'}
         self.register_uris([
             dict(method=method, uri=self.object_endpoint,
                  headers=self.headers,
@@ -140,7 +139,7 @@ class TestObject(base_test_object.BaseTestObject):
                      headers=sent_headers))
         ])
 
-        rv = sot.create(self.conn.object_store)
+        rv = sot.create(self.cloud.object_store)
         self.assertEqual(rv.etag, self.headers['Etag'])
 
         self.assert_calls()
